"use strict";
// Copyright (c) Vidar Tonaas Fauske.
// Distributed under the terms of the Modified BSD License.
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var base_1 = require("@jupyter-widgets/base");
var chromabar_1 = require("chromabar");
var d3_selection_1 = require("d3-selection");
var version_1 = require("../version");
var ColorBarModel = /** @class */ (function (_super) {
    __extends(ColorBarModel, _super);
    function ColorBarModel() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ColorBarModel.prototype.defaults = function () {
        return __assign({}, _super.prototype.defaults.call(this), { _model_name: ColorBarModel.model_name, _model_module: ColorBarModel.model_module, _model_module_version: ColorBarModel.model_module_version, _view_name: ColorBarModel.view_name, _view_module: ColorBarModel.view_module, _view_module_version: ColorBarModel.view_module_version, colormap: null, orientation: 'vertical', side: 'bottomright', length: 100, breadth: 30, border_thickness: 1, title: null, title_padding: 30, axis_padding: null });
    };
    ColorBarModel.prototype.initialize = function (attributes, options) {
        _super.prototype.initialize.call(this, attributes, options);
        this.setupListeners();
    };
    ColorBarModel.prototype.setupListeners = function () {
        var _this = this;
        // register listener for current child value
        var childAttrName = 'colormap';
        var curValue = this.get(childAttrName);
        if (curValue) {
            this.listenTo(curValue, 'change', this.onChildChanged.bind(this));
            this.listenTo(curValue, 'childchange', this.onChildChanged.bind(this));
        }
        // make sure to (un)hook listeners when child points to new object
        this.on("change:" + childAttrName, function (model, value) {
            var prevModel = _this.previous(childAttrName);
            var currModel = value;
            if (prevModel) {
                _this.stopListening(prevModel);
            }
            if (currModel) {
                _this.listenTo(currModel, 'change', _this.onChildChanged.bind(_this));
                _this.listenTo(currModel, 'childchange', _this.onChildChanged.bind(_this));
            }
        }, this);
    };
    ColorBarModel.prototype.onChildChanged = function (model) {
        // Propagate up hierarchy:
        this.trigger('childchange', this);
    };
    ColorBarModel.serializers = __assign({}, base_1.DOMWidgetModel.serializers, { colormap: { deserialize: base_1.unpack_models } });
    ColorBarModel.model_name = 'ColorBarModel';
    ColorBarModel.model_module = version_1.MODULE_NAME;
    ColorBarModel.model_module_version = version_1.JUPYTER_EXTENSION_VERSION;
    ColorBarModel.view_name = 'ColorBarView';
    ColorBarModel.view_module = version_1.MODULE_NAME;
    ColorBarModel.view_module_version = version_1.JUPYTER_EXTENSION_VERSION;
    return ColorBarModel;
}(base_1.DOMWidgetModel));
exports.ColorBarModel = ColorBarModel;
var ColorBarView = /** @class */ (function (_super) {
    __extends(ColorBarView, _super);
    function ColorBarView() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ColorBarView.prototype.render = function () {
        this.onChange();
        this.model.on('change', this.onChange, this);
        this.model.on('childchange', this.onChange, this);
    };
    ColorBarView.prototype.onChange = function () {
        var cmModel = this.model.get('colormap');
        var horizontal = this.model.get('orientation') === 'horizontal';
        var barFunc = chromabar_1.chromabar(cmModel.obj)
            .orientation(this.model.get('orientation'))
            .side(this.model.get('side'))
            .barLength(this.model.get('length'))
            .breadth(this.model.get('breadth'))
            .borderThickness(this.model.get('border_thickness'))
            .title(this.model.get('title'))
            .titlePadding(this.model.get('title_padding'))
            .axisPadding(this.model.get('axis_padding'));
        var svg = d3_selection_1.select(this.el)
            .selectAll('svg.jupyterColorbar').data([null]);
        svg = svg.merge(svg.enter().append('svg')
            .attr('class', 'jupyterColorbar'));
        svg
            .attr('height', barFunc.minHeight() + 10)
            .attr('width', barFunc.minWidth() + (horizontal ? 30 : 10));
        var g = svg.selectAll('g').data([null]);
        g = g.merge(g.enter().append('g'));
        g
            .attr('transform', 'translate(5, 5)')
            .call(barFunc);
    };
    return ColorBarView;
}(base_1.DOMWidgetView));
exports.ColorBarView = ColorBarView;
//# sourceMappingURL=colorbar.js.map