"use strict";
// Copyright (c) Vidar Tonaas Fauske.
// Distributed under the terms of the Modified BSD License.
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var base_1 = require("@jupyter-widgets/base");
var chromabar_1 = require("chromabar");
var d3_selection_1 = require("d3-selection");
var version_1 = require("../version");
var ColorMapEditorModel = /** @class */ (function (_super) {
    __extends(ColorMapEditorModel, _super);
    function ColorMapEditorModel() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ColorMapEditorModel.prototype.defaults = function () {
        return __assign({}, _super.prototype.defaults.call(this), { _model_name: ColorMapEditorModel.model_name, _model_module: ColorMapEditorModel.model_module, _model_module_version: ColorMapEditorModel.model_module_version, _view_name: ColorMapEditorModel.view_name, _view_module: ColorMapEditorModel.view_module, _view_module_version: ColorMapEditorModel.view_module_version, colormap: null, orientation: 'horizontal', length: 300, breadth: 30, border_thickness: 1 });
    };
    ColorMapEditorModel.prototype.initialize = function (attributes, options) {
        _super.prototype.initialize.call(this, attributes, options);
        this.setupListeners();
    };
    ColorMapEditorModel.prototype.setupListeners = function () {
        var _this = this;
        // register listener for current child value
        var childAttrName = 'colormap';
        var curValue = this.get(childAttrName);
        if (curValue) {
            this.listenTo(curValue, 'change', this.onChildChanged.bind(this));
            this.listenTo(curValue, 'childchange', this.onChildChanged.bind(this));
        }
        // make sure to (un)hook listeners when child points to new object
        this.on("change:" + childAttrName, function (model, value) {
            var prevModel = _this.previous(childAttrName);
            var currModel = value;
            if (prevModel) {
                _this.stopListening(prevModel);
            }
            if (currModel) {
                _this.listenTo(currModel, 'change', _this.onChildChanged.bind(_this));
                _this.listenTo(currModel, 'childchange', _this.onChildChanged.bind(_this));
            }
        }, this);
    };
    ColorMapEditorModel.prototype.onChildChanged = function (model) {
        // Propagate up hierarchy:
        this.trigger('childchange', this);
    };
    ColorMapEditorModel.serializers = __assign({}, base_1.DOMWidgetModel.serializers, { colormap: { deserialize: base_1.unpack_models } });
    ColorMapEditorModel.model_name = 'ColorMapEditorModel';
    ColorMapEditorModel.model_module = version_1.MODULE_NAME;
    ColorMapEditorModel.model_module_version = version_1.JUPYTER_EXTENSION_VERSION;
    ColorMapEditorModel.view_name = 'ColorMapEditorView';
    ColorMapEditorModel.view_module = version_1.MODULE_NAME;
    ColorMapEditorModel.view_module_version = version_1.JUPYTER_EXTENSION_VERSION;
    return ColorMapEditorModel;
}(base_1.DOMWidgetModel));
exports.ColorMapEditorModel = ColorMapEditorModel;
var ColorMapEditorView = /** @class */ (function (_super) {
    __extends(ColorMapEditorView, _super);
    function ColorMapEditorView() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ColorMapEditorView.prototype.render = function () {
        this.onChange();
        this.model.on('change', this.onChange, this);
        this.model.on('childchange', this.onChange, this);
    };
    ColorMapEditorView.prototype.onChange = function () {
        var cmModel = this.model.get('colormap');
        var horizontal = this.model.get('orientation') === 'horizontal';
        var editorFn = chromabar_1.chromaEditor(cmModel.obj)
            .orientation(this.model.get('orientation'))
            .barLength(this.model.get('length'))
            .breadth(this.model.get('breadth'))
            .borderThickness(this.model.get('border_thickness'))
            .onUpdate(function (save) {
            // Sync back all changes to both server and here
            cmModel.syncToModel({});
            if (save) {
                cmModel.save_changes();
            }
        });
        var svg = d3_selection_1.select(this.el)
            .selectAll('svg.jupyterColorbar').data([null]);
        svg = svg.merge(svg.enter().append('svg')
            .attr('class', 'jupyterColorbar'));
        svg
            .attr('height', editorFn.minHeight() + (horizontal ? 20 : 10))
            .attr('width', editorFn.minWidth() + (horizontal ? 10 : 20));
        var g = svg.selectAll('g').data([null]);
        g = g.merge(g.enter().append('g'));
        g
            .attr('transform', "translate(" + (horizontal ? 10 : 5) + ", " + (horizontal ? 5 : 10) + ")")
            .call(editorFn);
    };
    return ColorMapEditorView;
}(base_1.DOMWidgetView));
exports.ColorMapEditorView = ColorMapEditorView;
//# sourceMappingURL=editor.js.map