/// <reference types="backbone" />
import { TypedArray } from 'jupyter-dataserializers';
import { LinearScaleModel, LogScaleModel } from '../continuous';
import { SequentialScaleModel } from '../scale';
/**
 * Contiguous color map.
 */
export declare class LinearColorScaleModel extends LinearScaleModel {
    defaults(): any;
    static model_name: string;
}
/**
 * Contiguous color map.
 */
export declare class LogColorScaleModel extends LogScaleModel {
    defaults(): any;
    static model_name: string;
}
export declare type ColorInterpolator = (t: number) => string;
/**
 * A contiguous color map created from a named color map.
 */
declare class NamedColorMapBase extends SequentialScaleModel<string> {
    getInterpolatorFactory(): ColorInterpolator;
    getInterpolatorFactoryName(): string | null;
    constructObject(): import("d3-scale").ScaleSequential<string>;
    /**
     * Sync the model properties to the d3 object.
     */
    syncToObject(): void;
    syncToModel(toSet: Backbone.ObjectHash): void;
}
/**
 * A contiguous color map created from a named color map.
 */
export declare class NamedSequentialColorMap extends NamedColorMapBase {
    defaults(): any;
    static model_name: string;
}
/**
 * A contiguous color map created from a named color map.
 */
export declare class NamedDivergingColorMap extends NamedColorMapBase {
    defaults(): any;
    static model_name: string;
}
export interface ColorScale {
    copy(): this;
    domain(domain: [number, number]): this;
    (value: number | {
        valueOf(): number;
    }): string;
}
export interface ColorMapModel {
    obj: ColorScale;
}
export declare function colormapAsRGBArray(mapModel: ColorMapModel, size: number): Uint8ClampedArray;
export declare function colormapAsRGBArray<T extends TypedArray>(mapModel: ColorMapModel, array: T): T;
export declare function colormapAsRGBAArray(mapModel: ColorMapModel, size: number): Uint8ClampedArray;
export declare function colormapAsRGBAArray<T extends TypedArray>(mapModel: ColorMapModel, array: T): T;
export {};
