"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var d3_color_1 = require("d3-color");
var d3_scale_1 = require("d3-scale");
var d3Chromatic = require("d3-scale-chromatic");
var continuous_1 = require("../continuous");
var scale_1 = require("../scale");
/**
 * Contiguous color map.
 */
var LinearColorScaleModel = /** @class */ (function (_super) {
    __extends(LinearColorScaleModel, _super);
    function LinearColorScaleModel() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    LinearColorScaleModel.prototype.defaults = function () {
        return __assign({}, _super.prototype.defaults.call(this), { range: ['black', 'white'] });
    };
    LinearColorScaleModel.model_name = 'LinearColorScaleModel';
    return LinearColorScaleModel;
}(continuous_1.LinearScaleModel));
exports.LinearColorScaleModel = LinearColorScaleModel;
/**
 * Contiguous color map.
 */
var LogColorScaleModel = /** @class */ (function (_super) {
    __extends(LogColorScaleModel, _super);
    function LogColorScaleModel() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    LogColorScaleModel.prototype.defaults = function () {
        return __assign({}, _super.prototype.defaults.call(this), { range: ['black', 'white'] });
    };
    LogColorScaleModel.model_name = 'LogColorScaleModel';
    return LogColorScaleModel;
}(continuous_1.LogScaleModel));
exports.LogColorScaleModel = LogColorScaleModel;
var chromaticInterpLut = {};
for (var _i = 0, _a = Object.keys(d3Chromatic); _i < _a.length; _i++) {
    var key = _a[_i];
    if (key.indexOf('interpolate') === 0) {
        var lowKey = key.slice('interpolate'.length).toLowerCase();
        chromaticInterpLut[lowKey] = d3Chromatic[key];
    }
}
/**
 * A contiguous color map created from a named color map.
 */
var NamedColorMapBase = /** @class */ (function (_super) {
    __extends(NamedColorMapBase, _super);
    function NamedColorMapBase() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NamedColorMapBase.prototype.getInterpolatorFactory = function () {
        var name = this.get('name');
        return chromaticInterpLut[name.toLowerCase()];
    };
    NamedColorMapBase.prototype.getInterpolatorFactoryName = function () {
        var interp = this.obj.interpolator();
        // Do a reverse lookup in d3Chromatic
        var lut = d3Chromatic;
        for (var _i = 0, _a = Object.keys(lut); _i < _a.length; _i++) {
            var key = _a[_i];
            if (interp === lut[key]) {
                var name_1 = key.replace(/^interpolate/, '');
                return name_1;
            }
        }
        throw new Error("Unknown color interpolator name of function: " + interp);
    };
    NamedColorMapBase.prototype.constructObject = function () {
        var interpolator = this.getInterpolatorFactory();
        return d3_scale_1.scaleSequential(interpolator);
    };
    /**
     * Sync the model properties to the d3 object.
     */
    NamedColorMapBase.prototype.syncToObject = function () {
        _super.prototype.syncToObject.call(this);
        var interpolator = this.getInterpolatorFactory();
        this.obj
            .interpolator(interpolator);
    };
    NamedColorMapBase.prototype.syncToModel = function (toSet) {
        toSet['name'] = this.getInterpolatorFactoryName();
        _super.prototype.syncToModel.call(this, toSet);
    };
    return NamedColorMapBase;
}(scale_1.SequentialScaleModel));
/**
 * A contiguous color map created from a named color map.
 */
var NamedSequentialColorMap = /** @class */ (function (_super) {
    __extends(NamedSequentialColorMap, _super);
    function NamedSequentialColorMap() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NamedSequentialColorMap.prototype.defaults = function () {
        return __assign({}, _super.prototype.defaults.call(this), { name: 'Viridis' });
    };
    NamedSequentialColorMap.model_name = 'NamedSequentialColorMap';
    return NamedSequentialColorMap;
}(NamedColorMapBase));
exports.NamedSequentialColorMap = NamedSequentialColorMap;
/**
 * A contiguous color map created from a named color map.
 */
var NamedDivergingColorMap = /** @class */ (function (_super) {
    __extends(NamedDivergingColorMap, _super);
    function NamedDivergingColorMap() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NamedDivergingColorMap.prototype.defaults = function () {
        return __assign({}, _super.prototype.defaults.call(this), { name: 'BrBG' });
    };
    NamedDivergingColorMap.model_name = 'NamedDivergingColorMap';
    return NamedDivergingColorMap;
}(NamedColorMapBase));
exports.NamedDivergingColorMap = NamedDivergingColorMap;
function colormapAsRGBArray(mapModel, data) {
    var n;
    if (typeof data === 'number') {
        n = data;
        data = new Uint8ClampedArray(n * 3);
    }
    else {
        n = data.length / 3;
    }
    var scale = mapModel.obj.copy().domain([0, n]);
    for (var i = 0; i < n; ++i) {
        var color = d3_color_1.rgb(scale(i));
        data[i * 3 + 0] = color.r;
        data[i * 3 + 1] = color.g;
        data[i * 3 + 2] = color.b;
    }
    return data;
}
exports.colormapAsRGBArray = colormapAsRGBArray;
function colormapAsRGBAArray(mapModel, data) {
    var n;
    if (typeof data === 'number') {
        n = data;
        data = new Uint8ClampedArray(n * 4);
    }
    else {
        n = data.length / 4;
    }
    var scale = mapModel.obj.copy().domain([0, n]);
    for (var i = 0; i < n; ++i) {
        var color = d3_color_1.rgb(scale(i));
        data[i * 4 + 0] = color.r;
        data[i * 4 + 1] = color.g;
        data[i * 4 + 2] = color.b;
        data[i * 4 + 3] = color.opacity;
    }
    return data;
}
exports.colormapAsRGBAArray = colormapAsRGBAArray;
//# sourceMappingURL=scales.js.map