"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var d3_scale_1 = require("d3-scale");
var d3Interpolate = require("d3-interpolate");
var scale_1 = require("./scale");
/**
 * Find the name of the d3-interpolate function
 */
function interpolatorName(fun) {
    for (var _i = 0, _a = Object.keys(d3Interpolate); _i < _a.length; _i++) {
        var key = _a[_i];
        if (d3Interpolate[key] === fun) {
            return key;
        }
    }
    throw ReferenceError("Cannot find name of interpolator " + fun);
}
/**
 * A widget model of a continuous scale
 */
var ContinuousScaleModel = /** @class */ (function (_super) {
    __extends(ContinuousScaleModel, _super);
    function ContinuousScaleModel() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ContinuousScaleModel.prototype.defaults = function () {
        return __assign({}, _super.prototype.defaults.call(this), { domain: [0, 1], range: [0, 1], interpolator: 'interpolate', clamp: false });
    };
    ContinuousScaleModel.prototype.createPropertiesArrays = function () {
        _super.prototype.createPropertiesArrays.call(this);
        this.simpleProperties.push('domain', 'range', 'clamp');
    };
    /**
     * Sync the model properties to the d3 object.
     */
    ContinuousScaleModel.prototype.syncToObject = function () {
        _super.prototype.syncToObject.call(this);
        var interpolatorName = this.get('interpolator') || 'interpolate';
        var interpolator = d3Interpolate[interpolatorName];
        this.obj.interpolate(interpolator);
    };
    /**
     * Synt the d3 object properties to the model.
     */
    ContinuousScaleModel.prototype.syncToModel = function (toSet) {
        var interpolator = this.obj.interpolate();
        toSet['interpolator'] = interpolatorName(interpolator);
        _super.prototype.syncToModel.call(this, toSet);
    };
    ContinuousScaleModel.serializers = __assign({}, scale_1.ScaleModel.serializers);
    return ContinuousScaleModel;
}(scale_1.ScaleModel));
exports.ContinuousScaleModel = ContinuousScaleModel;
/**
 * A widget model of a linear scale
 */
var LinearScaleModel = /** @class */ (function (_super) {
    __extends(LinearScaleModel, _super);
    function LinearScaleModel() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * Create the wrapped d3-scale scaleLinear object
     */
    LinearScaleModel.prototype.constructObject = function () {
        return d3_scale_1.scaleLinear();
    };
    LinearScaleModel.serializers = __assign({}, ContinuousScaleModel.serializers);
    LinearScaleModel.model_name = 'LinearScaleModel';
    return LinearScaleModel;
}(ContinuousScaleModel));
exports.LinearScaleModel = LinearScaleModel;
/**
 * A widget model of a linear scale
 */
var LogScaleModel = /** @class */ (function (_super) {
    __extends(LogScaleModel, _super);
    function LogScaleModel() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    LogScaleModel.prototype.defaults = function () {
        return __assign({}, _super.prototype.defaults.call(this), { base: 10 });
    };
    LogScaleModel.prototype.createPropertiesArrays = function () {
        _super.prototype.createPropertiesArrays.call(this);
        this.simpleProperties.push('base');
    };
    /**
     * Create the wrapped d3-scale scaleLinear object
     */
    LogScaleModel.prototype.constructObject = function () {
        return d3_scale_1.scaleLog();
    };
    LogScaleModel.serializers = __assign({}, ContinuousScaleModel.serializers);
    LogScaleModel.model_name = 'LogScaleModel';
    return LogScaleModel;
}(ContinuousScaleModel));
exports.LogScaleModel = LogScaleModel;
/**
 * A widget model of a linear scale
 */
var PowScaleModel = /** @class */ (function (_super) {
    __extends(PowScaleModel, _super);
    function PowScaleModel() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    PowScaleModel.prototype.defaults = function () {
        this.constructor;
        return __assign({}, _super.prototype.defaults.call(this), { exponent: 1 });
    };
    PowScaleModel.prototype.createPropertiesArrays = function () {
        _super.prototype.createPropertiesArrays.call(this);
        this.simpleProperties.push('exponent');
    };
    /**
     * Create the wrapped d3-scale scaleLinear object
     */
    PowScaleModel.prototype.constructObject = function () {
        return d3_scale_1.scalePow();
    };
    PowScaleModel.serializers = __assign({}, ContinuousScaleModel.serializers);
    PowScaleModel.model_name = 'PowScaleModel';
    return PowScaleModel;
}(ContinuousScaleModel));
exports.PowScaleModel = PowScaleModel;
//# sourceMappingURL=continuous.js.map