"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var base_1 = require("@jupyter-widgets/base");
var d3_scale_1 = require("d3-scale");
var jupyter_dataserializers_1 = require("jupyter-dataserializers");
var version_1 = require("./version");
/**
 * Base model for scales
 */
var ScaleModel = /** @class */ (function (_super) {
    __extends(ScaleModel, _super);
    function ScaleModel() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * Returns default values for the model attributes.
     */
    ScaleModel.prototype.defaults = function () {
        var ctor = this.constructor;
        return __assign({}, _super.prototype.defaults.call(this), { _model_name: ctor.model_name, _model_module: ctor.model_module, _model_module_version: ctor.model_module_version, _view_name: ctor.view_name, _view_module: ctor.view_module, _view_module_version: ctor.view_module_version });
    };
    /**
     * Backbone initialize function.
     */
    ScaleModel.prototype.initialize = function (attributes, options) {
        var _this = this;
        _super.prototype.initialize.call(this, attributes, options);
        this.createPropertiesArrays();
        // Instantiate scale object
        this.initPromise = this.createObject().then(function () {
            // sync the properties from the server to the model
            _this.syncToObject();
            // sync any properties that might have mutated back
            _this.syncToModel({});
            // setup msg, model, and children change listeners
            _this.setupListeners();
        });
    };
    ScaleModel.prototype.createPropertiesArrays = function () {
        this.datawidgetProperties = [];
        this.childModelProperties = [];
        this.simpleProperties = [];
    };
    /**
     * Update the model attributes from the objects properties.
     *
     * The base method calls `this.set(toSet, 'pushFromObject');`.
     * Overriding methods should add any properties not in
     * simpleProperties to the hash before calling the super
     * method.
     */
    ScaleModel.prototype.syncToModel = function (toSet) {
        for (var _i = 0, _a = this.simpleProperties; _i < _a.length; _i++) {
            var name_1 = _a[_i];
            toSet[name_1] = this.obj[name_1]();
        }
        // Apply all direct changes at once
        this.set(toSet, 'pushFromObject');
    };
    /**
     * Update the model attributes from the objects properties.
     */
    ScaleModel.prototype.syncToObject = function () {
        // Sync the simple properties:
        for (var _i = 0, _a = this.simpleProperties; _i < _a.length; _i++) {
            var name_2 = _a[_i];
            this.obj[name_2](this.get(name_2));
        }
    };
    ;
    /**
     * Create or return the underlying object this model represents.
     */
    ScaleModel.prototype.createObject = function () {
        // call constructor method overridden by every class
        var objPromise = Promise.resolve(this.constructObject());
        return objPromise.then(this.processNewObj.bind(this));
    };
    /**
     * Process a new underlying object to represent this model.
     *
     * The base implementation sets up mapping between the model,
     * the cache, and the widget manager.
     */
    ScaleModel.prototype.processNewObj = function (obj) {
        obj.ipymodelId = this.model_id; // brand that sucker
        obj.ipymodel = this;
        this.obj = obj;
        return obj;
    };
    /**
     * Set up any event listeners.
     *
     * Called after object initialization is complete.
     */
    ScaleModel.prototype.setupListeners = function () {
        var _this = this;
        var _loop_1 = function (propName) {
            // register listener for current child value
            curValue = this_1.get(propName);
            if (curValue) {
                this_1.listenTo(curValue, 'change', this_1.onChildChanged.bind(this_1));
                this_1.listenTo(curValue, 'childchange', this_1.onChildChanged.bind(this_1));
            }
            // make sure to (un)hook listeners when child points to new object
            this_1.on('change:' + propName, function (model, value, options) {
                var prevModel = _this.previous(propName);
                var currModel = value;
                if (prevModel) {
                    _this.stopListening(prevModel);
                }
                if (currModel) {
                    _this.listenTo(currModel, 'change', _this.onChildChanged.bind(_this));
                    _this.listenTo(currModel, 'childchange', _this.onChildChanged.bind(_this));
                }
            }, this_1);
        };
        var this_1 = this, curValue;
        // Handle changes in child model instance props
        for (var _i = 0, _a = this.childModelProperties; _i < _a.length; _i++) {
            var propName = _a[_i];
            _loop_1(propName);
        }
        ;
        // Handle changes in data widgets/union properties
        for (var _b = 0, _c = this.datawidgetProperties; _b < _c.length; _b++) {
            var propName = _c[_b];
            jupyter_dataserializers_1.listenToUnion(this, propName, this.onChildChanged.bind(this), false);
        }
        ;
        this.on('change', this.onChange, this);
        this.on('msg:custom', this.onCustomMessage, this);
    };
    ScaleModel.prototype.onChange = function (model, options) {
        if (options !== 'pushFromObject') {
            this.syncToObject();
        }
    };
    ScaleModel.prototype.onChildChanged = function (model, options) {
        // Propagate up hierarchy:
        this.trigger('childchange', this);
    };
    ScaleModel.prototype.onCustomMessage = function (content, buffers) {
    };
    ScaleModel.serializers = base_1.WidgetModel.serializers;
    ScaleModel.model_module = version_1.MODULE_NAME;
    ScaleModel.model_module_version = version_1.JUPYTER_EXTENSION_VERSION;
    ScaleModel.view_name = null;
    ScaleModel.view_module = null;
    ScaleModel.view_module_version = version_1.JUPYTER_EXTENSION_VERSION;
    return ScaleModel;
}(base_1.WidgetModel));
exports.ScaleModel = ScaleModel;
/**
 * A widget model of a sequential scale
 */
var SequentialScaleModel = /** @class */ (function (_super) {
    __extends(SequentialScaleModel, _super);
    function SequentialScaleModel() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SequentialScaleModel.prototype.defaults = function () {
        return __assign({}, _super.prototype.defaults.call(this), { domain: [0, 1], clamp: false });
    };
    SequentialScaleModel.prototype.createPropertiesArrays = function () {
        _super.prototype.createPropertiesArrays.call(this);
        this.simpleProperties.push('domain', 'clamp');
    };
    SequentialScaleModel.serializers = __assign({}, ScaleModel.serializers);
    return SequentialScaleModel;
}(ScaleModel));
exports.SequentialScaleModel = SequentialScaleModel;
/**
 * A widget model of a quantize scale
 */
var QuantizeScaleModel = /** @class */ (function (_super) {
    __extends(QuantizeScaleModel, _super);
    function QuantizeScaleModel() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    QuantizeScaleModel.prototype.defaults = function () {
        return __assign({}, _super.prototype.defaults.call(this), { domain: [0, 1], range: [0, 1] });
    };
    QuantizeScaleModel.prototype.createPropertiesArrays = function () {
        _super.prototype.createPropertiesArrays.call(this);
        this.simpleProperties.push('domain', 'range');
    };
    QuantizeScaleModel.prototype.constructObject = function () {
        return d3_scale_1.scaleQuantize();
    };
    QuantizeScaleModel.serializers = __assign({}, ScaleModel.serializers);
    QuantizeScaleModel.model_name = 'QuantizeScaleModel';
    return QuantizeScaleModel;
}(ScaleModel));
exports.QuantizeScaleModel = QuantizeScaleModel;
/**
 * A widget model of a quantile scale
 */
var QuantileScaleModel = /** @class */ (function (_super) {
    __extends(QuantileScaleModel, _super);
    function QuantileScaleModel() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    QuantileScaleModel.prototype.defaults = function () {
        return __assign({}, _super.prototype.defaults.call(this), { domain: [0], range: [0] });
    };
    QuantileScaleModel.prototype.createPropertiesArrays = function () {
        _super.prototype.createPropertiesArrays.call(this);
        this.simpleProperties.push('domain', 'range');
    };
    QuantileScaleModel.prototype.constructObject = function () {
        return d3_scale_1.scaleQuantile();
    };
    QuantileScaleModel.serializers = __assign({}, ScaleModel.serializers);
    QuantileScaleModel.model_name = 'QuantileScaleModel';
    return QuantileScaleModel;
}(ScaleModel));
exports.QuantileScaleModel = QuantileScaleModel;
/**
 * A widget model of a teshold scale
 */
var TresholdScaleModel = /** @class */ (function (_super) {
    __extends(TresholdScaleModel, _super);
    function TresholdScaleModel() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    TresholdScaleModel.prototype.defaults = function () {
        return __assign({}, _super.prototype.defaults.call(this), { domain: [], range: [0] });
    };
    TresholdScaleModel.prototype.createPropertiesArrays = function () {
        _super.prototype.createPropertiesArrays.call(this);
        this.simpleProperties.push('domain', 'range');
    };
    TresholdScaleModel.prototype.constructObject = function () {
        return d3_scale_1.scaleQuantile();
    };
    TresholdScaleModel.serializers = __assign({}, ScaleModel.serializers);
    TresholdScaleModel.model_name = 'TresholdScaleModel';
    return TresholdScaleModel;
}(ScaleModel));
exports.TresholdScaleModel = TresholdScaleModel;
/**
 * A widget model of an ordinal scale
 */
var OrdinalScaleModel = /** @class */ (function (_super) {
    __extends(OrdinalScaleModel, _super);
    function OrdinalScaleModel() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    OrdinalScaleModel.prototype.defaults = function () {
        return __assign({}, _super.prototype.defaults.call(this), { domain: [], range: [], unknown: undefined });
    };
    OrdinalScaleModel.prototype.createPropertiesArrays = function () {
        _super.prototype.createPropertiesArrays.call(this);
        this.simpleProperties.push('domain', 'range');
    };
    OrdinalScaleModel.prototype.constructObject = function () {
        return d3_scale_1.scaleOrdinal();
    };
    OrdinalScaleModel.prototype.syncToObject = function () {
        _super.prototype.syncToObject.call(this);
        var unknown = this.get('unknown');
        if (unknown === undefined) {
            unknown = d3_scale_1.scaleImplicit;
        }
        this.obj.unknown(unknown);
    };
    OrdinalScaleModel.prototype.syncToModel = function (toSet) {
        var unknown = this.obj.unknown();
        if (unknown === d3_scale_1.scaleImplicit) {
            unknown = undefined;
        }
        toSet['unknown'] = unknown;
        _super.prototype.syncToModel.call(this, toSet);
    };
    OrdinalScaleModel.serializers = __assign({}, ScaleModel.serializers);
    OrdinalScaleModel.model_name = 'OrdinalScaleModel';
    return OrdinalScaleModel;
}(ScaleModel));
exports.OrdinalScaleModel = OrdinalScaleModel;
//# sourceMappingURL=scale.js.map